(in-package #:cl-user)
(defpackage #:cli
  (:use #:cl)
  (:import-from #:clingon
                #:make-command
                #:make-option
                #:run)
  ;; (:import-from #:com.inoue.jzon
  ;;               #:parse
  ;;               #:stringify)
  (:import-from #:utils
                #:make-config-file
                #:read-config-file
                #:send-notification
                #:update-ip-addresses)
  ;; (:local-nicknames (#:jzon #:com.inuoe.jzon))
  (:export #:main)
  (:documentation "Package for updating the DNS records for domain names."))
(in-package #:cli)

;;(defparameter *app* (top-level/command))

(defun main ()
  "The main entrypoint of our CLI program"
  (let ((app (top-level/command)))
    (clingon:run app)))

(defun top-level/command ()
  "Creates and returns the top-level command"
  (clingon:make-command
   :name "dup"
   :description "Update IP addresses associated with your DNS records."
   :version "0.0.1"
   :license "MIT"
   :authors '("Craig Oates <craig@craigoates.net>")
   :usage "[-v] [-c <PATH>] [-m <PATH>] [-t]"
   :options (top-level/options)
   :handler #'top-level/handler))

(defun top-level/options ()
  "Creates and returns the options for the top-level command"
  (list
   (clingon:make-option
    :string
    :description "Load a config. file at specified path."
    :short-name #\c
    :long-name "config"
    ; :initial-value #P"~/ddns-updater/ddns-updater.json"
    ;; :env-vars '("DDNS-CONF")
    :key :config)
   (clingon:make-option
    :string
    :description "Make a config. file at specified path."
    :short-name #\m
    :long-name "make"
    ;; :initial-value #P"~/ddns-updater/ddns-updater.json"
    :key :make)
   (clingon:make-option
    :boolean
    :description "Test Telegram notification system."
    :short-name #\t
    :long-name "telegram-test"
    :key :telegram-test)
   (clingon:make-option
    :counter
    :description "Verbosity level."
    :short-name #\v
    :long-name "verbose"
    :key :verbose)))

(defun top-level/handler (cmd)
  "The top-level handler.
If no configuration file (path) is provided, the program will use the default
one (hard-coded into this function."
  (let ((config (clingon:getopt cmd :config))
        (make (clingon:getopt cmd :make))
        (telegram-test (clingon:getopt cmd :telegram-test))
        (verbose (clingon:getopt cmd :verbose)))
    (when (> verbose 0)
      (format t "Configuration File: ~A~%"
              (if (null config) "~/ddns-updater.json" config))
      (format t "Make File: ~A~%" (if (null make) "N/A" make))
      (format t "Telegram Test: ~A~%" (if (null telegram-test) "No" "Yes"))
      (format t "Verbosity Level: ~A~%" verbose))
    ;; Load configuration file.
    (let ((conf-data (if config
                         (utils:read-config-file config)
                         (utils:read-config-file #P"~/ddns-updater.json"))))
      ;; Update DNS records.
      (utils:update-ip-addresses
       conf-data (if config config #P"~/ddns-updater.json"))
      ;; Make configuration file.
      (when make
        (utils:make-config-file make)
        (when (> verbose 0) (log:info "Made config. file: ~a." make)))
      ;; Telegram Test
      (when telegram-test
        (utils:send-notification
         (gethash "telegram-token" conf-data)
         (gethash "chat-id" conf-data)
         (format nil
                 "[🔷 TEST] This is a test notification. Sent from ~a."
                 (gethash "server-name" conf-data)))))))
